# -*- coding: utf-8 -*-
"""Bencode plugin related functions and classes for testing."""

from tests.parsers import test_lib


class BencodePluginTestCase(test_lib.ParserTestCase):
  """Bencode parser plugin test case."""
